'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var useSWR = require('swr');
var useSWR__default = _interopDefault(useSWR);
var http = require('@superdispatch/http');
var hooks = require('@superdispatch/hooks');

function inputToArgs(input) {
  return typeof input === 'string' ? [input] : input;
}
function argsToKey(template, options) {
  var {
    url,
    body,
    method
  } = http.parseHTTPEndpoint(template, options);
  return [method, url, typeof body == 'string' ? body : undefined, options == null ? void 0 : options.key];
}
function inputToKey(input) {
  var [template, options] = inputToArgs(input);
  return argsToKey(template, options);
}

function revalidateHTTPResource(input) {
  return useSWR.mutate(inputToKey(input)).then(() => void 0);
}
function mutateHTTPResource(input, fn, shouldRevalidate) {
  return useSWR.mutate(inputToKey(input), fn, shouldRevalidate);
}
function clearHTTPResourceCache() {
  useSWR.cache.clear();
}
function readCachedHTTPResource(input) {
  return useSWR.cache.get(inputToKey(input));
}

function useHTTPInfiniteResource(input, makeParams, fetcher, resourceOptions) {
  var [template, baseParams] = inputToArgs(input);
  return useSWR.useSWRInfinite((index, prev) => {
    var params = makeParams(index, prev);

    if (!params) {
      return null;
    }

    return argsToKey(template, Object.assign({}, baseParams, params));
  }, (method, url, body) => {
    var options = Object.assign({}, baseParams);

    if (body != null) {
      options.body = body;
    }

    return fetcher(method + " " + url, options);
  }, resourceOptions);
}

function useHTTPResource(input, fetcher, options) {
  var [key, template, params] = hooks.useDeepEqualMemo(() => {
    if (input == null) {
      return [null];
    }

    var [nextTemplate, nextParams] = inputToArgs(input);
    var nextKey = argsToKey(nextTemplate, nextParams);
    return [nextKey, nextTemplate, nextParams];
  }, [input]);
  return useSWR__default(key, Object.assign({}, options, {
    fetcher: !template ? undefined : () => fetcher(template, params)
  }));
}

exports.clearHTTPResourceCache = clearHTTPResourceCache;
exports.mutateHTTPResource = mutateHTTPResource;
exports.readCachedHTTPResource = readCachedHTTPResource;
exports.revalidateHTTPResource = revalidateHTTPResource;
exports.useHTTPInfiniteResource = useHTTPInfiniteResource;
exports.useHTTPResource = useHTTPResource;
//# sourceMappingURL=index.js.map
