import { URITemplateParams } from '@superdispatch/uri';
import { HTTPEndpointParams } from '@superdispatch/http';
import { SWRInfiniteConfiguration, SWRInfiniteResponse, SWRConfiguration, SWRResponse } from 'swr';

declare type HTTPResourceKeyOption = bigint | boolean | null | number | string;
declare type HTTPResourceFetcherOptions<TParams extends URITemplateParams = URITemplateParams> = HTTPEndpointParams<{
    key?: HTTPResourceKeyOption;
} & TParams>;
declare type HTTPResourceFetcher<TData, TParams extends URITemplateParams = URITemplateParams> = (template: string, options?: HTTPResourceFetcherOptions<TParams>) => Promise<TData>;
declare type HTTPResourceFetcherArgs<TParams extends URITemplateParams = URITemplateParams> = readonly [template: string, options?: HTTPResourceFetcherOptions<TParams>];
declare type HTTPResourceInput<TParams extends URITemplateParams = URITemplateParams> = string | HTTPResourceFetcherArgs<TParams>;
declare type HTTPResourceKey = [
    method: string,
    url: string,
    body?: string,
    key?: HTTPResourceKeyOption
];

declare function revalidateHTTPResource<TParams extends URITemplateParams>(input: HTTPResourceInput<TParams>): Promise<void>;
declare function mutateHTTPResource<TData, TParams extends URITemplateParams = URITemplateParams>(input: HTTPResourceInput<TParams>, fn: (prev?: TData) => TData | undefined, shouldRevalidate?: boolean): Promise<void>;
declare function clearHTTPResourceCache(): void;
declare function readCachedHTTPResource<TData, TParams extends URITemplateParams = URITemplateParams>(input: HTTPResourceInput<TParams>): TData | undefined;

declare type HTTPInfiniteResourceOptions<TData, TError = Error> = Omit<SWRInfiniteConfiguration<TData, TError>, 'fetcher'>;
declare type HTTPInfiniteResourceParamFactory<TData, TParams extends URITemplateParams = URITemplateParams> = (index: number, prev: TData | null) => null | HTTPEndpointParams<TParams>;
declare type HTTPInfiniteResource<TData, TError = Error> = Omit<SWRInfiniteResponse<TData, TError>, 'mutate'>;
declare function useHTTPInfiniteResource<TData, TParams extends URITemplateParams = URITemplateParams, TError = Error>(input: HTTPResourceInput<TParams>, makeParams: HTTPInfiniteResourceParamFactory<TData, TParams>, fetcher: HTTPResourceFetcher<TData>, resourceOptions?: HTTPInfiniteResourceOptions<TData, TError>): HTTPInfiniteResource<TData, TError>;

declare type HTTPResourceOptions<TData, TError = Error> = Omit<SWRConfiguration<TData, TError>, 'fetcher'>;
declare type HTTPResource<TData, TError = Error> = SWRResponse<TData, TError>;
declare function useHTTPResource<TData, TParams extends URITemplateParams = URITemplateParams, TError = Error>(input: null | HTTPResourceInput<TParams>, fetcher: HTTPResourceFetcher<TData>, options?: HTTPResourceOptions<TData, TError>): HTTPResource<TData, TError>;

export { HTTPInfiniteResource, HTTPInfiniteResourceOptions, HTTPInfiniteResourceParamFactory, HTTPResource, HTTPResourceFetcher, HTTPResourceFetcherArgs, HTTPResourceFetcherOptions, HTTPResourceInput, HTTPResourceKey, HTTPResourceKeyOption, HTTPResourceOptions, clearHTTPResourceCache, mutateHTTPResource, readCachedHTTPResource, revalidateHTTPResource, useHTTPInfiniteResource, useHTTPResource };
