import useSWR, { mutate, cache, useSWRInfinite } from 'swr';
import { parseHTTPEndpoint } from '@superdispatch/http';
import { useDeepEqualMemo } from '@superdispatch/hooks';

function inputToArgs(input) {
  return typeof input === 'string' ? [input] : input;
}
function argsToKey(template, options) {
  var {
    url,
    body,
    method
  } = parseHTTPEndpoint(template, options);
  return [method, url, typeof body == 'string' ? body : undefined, options == null ? void 0 : options.key];
}
function inputToKey(input) {
  var [template, options] = inputToArgs(input);
  return argsToKey(template, options);
}

function revalidateHTTPResource(input) {
  return mutate(inputToKey(input)).then(() => void 0);
}
function mutateHTTPResource(input, fn, shouldRevalidate) {
  return mutate(inputToKey(input), fn, shouldRevalidate);
}
function clearHTTPResourceCache() {
  cache.clear();
}
function readCachedHTTPResource(input) {
  return cache.get(inputToKey(input));
}

function useHTTPInfiniteResource(input, makeParams, fetcher, resourceOptions) {
  var [template, baseParams] = inputToArgs(input);
  return useSWRInfinite((index, prev) => {
    var params = makeParams(index, prev);

    if (!params) {
      return null;
    }

    return argsToKey(template, Object.assign({}, baseParams, params));
  }, (method, url, body) => {
    var options = Object.assign({}, baseParams);

    if (body != null) {
      options.body = body;
    }

    return fetcher(method + " " + url, options);
  }, resourceOptions);
}

function useHTTPResource(input, fetcher, options) {
  var [key, template, params] = useDeepEqualMemo(() => {
    if (input == null) {
      return [null];
    }

    var [nextTemplate, nextParams] = inputToArgs(input);
    var nextKey = argsToKey(nextTemplate, nextParams);
    return [nextKey, nextTemplate, nextParams];
  }, [input]);
  return useSWR(key, Object.assign({}, options, {
    fetcher: !template ? undefined : () => fetcher(template, params)
  }));
}

export { clearHTTPResourceCache, mutateHTTPResource, readCachedHTTPResource, revalidateHTTPResource, useHTTPInfiniteResource, useHTTPResource };
//# sourceMappingURL=index.js.map
